function [bml,sml,niter] = ggd_ml(x,varargin)

% ML estimation of the shape parameter and the scale parameter for samples
% from a GGD. 

% INPUT
% x = samples
% tol = tolerance for Newton algorithm convergence (optional)
% maxiter = maximum number of iterations (optional)
% method = 'nr'     Newton Raphson (default),
%          'sec'    Secants,
%          'rf'     Regula Falsi,
%          'fast'   inetrpolation of mallat's ratio

% OUTPUT
% bml = ML estimate of the shape parameter
% sml = ML estimate of the variance
% niter = number of iterations
% bstart = initial guess for shape parameter
% reg = regularization of norm, so as to have differentiability in the
%       origin

% Author: Martino Alessandrini. Dec 2008

if (nargin < 2) || isempty(varargin{1}), ...
    tol = 10^-6; else tol = varargin{1}; end
if (nargin < 3) || isempty(varargin{2}), ...
    maxiter = 100; else maxiter = varargin{2}; end
if (nargin < 4) || isempty(varargin{3}), ...
    method = 'nr'; else method = varargin{3}; end
if (nargin < 5) || isempty(varargin{4}), ...
    bstart = [] ; else bstart = varargin{4}; end
if (nargin < 6) || isempty(varargin{5}), ...
    reg = 0 ; else reg = varargin{5}; end

x = x(:);
if ~isreal(x)
    x = [real(x);imag(x)];
end

x = x - mean(x); 
N = numel(x);

% -- initial guess (fast shape est)

if isempty(bstart)
    if reg == 0
        m1 = 1/N*sum(abs(x));
        m2 = 1/N*sum(x.^2);
        r = m1^2/m2;
        if r > 0.75
            bstart = 0.5;
        else
            bstart = abs(0.2718/(0.7697 - r) - 0.1247);
        end
    else
        bstart = 1;
    end
end


abs_x = sqrt(x.^2+reg^2);

%% -- fast method
if strcmp(method,'fast')
    bml = bstart;
    niter = 0;
end    
%% -- Newton Raphson method
if strcmp(method,'nr')
    adj = inf; % -- distance betwett successive solutions
    k = 0; % -- number of iterations
    bet = bstart;
    while (abs(adj) > tol)
        [g gp] = gfunc_int(bet,x,reg);
        adj = g/gp;
        bet = bet - adj;
        k = k + 1;
        if k == maxiter
            disp(['Convergence not reached in ' num2str(maxiter) 'iterations']);
            break
        end
    end
    bml = bet;
    niter = k;
end

%% -- Secants method
if strcmp(method,'sec')
    bvec(1) = bstart; % -- t-1 istant
    bvec(2) = bstart + 0.25; % -- t istant
    adj = inf;
    k = 0;
    while (abs(adj) > tol)
        ft = gfunc_int(bvec(2),x,reg);
        ftt = gfunc_int(bvec(1),x,reg);
        adj = (bvec(2)-bvec(1))/(ft - ftt)*ft;
        bvec(1) = bvec(2) - adj;
        bvec = circshift(bvec(:)',[0 1]);
        k = k + 1;
        if k == maxiter
            disp(['Convergence not reached in ' num2str(maxiter) 'iterations']);
            break
        end
    end
    bml = bvec(2);
    niter = k;
end
  
%% -- Regula Falsi
if strcmp(method,'rf')
    bvec(1) = bstart; % -- t-1 istant
    ftt = gfunc_int(bvec(1),x,reg); 
    pow = 1;
    dlt = 0.25;
    rf = 10;
    while rf >= 0 % -- initial couple
        bvec(2) = bvec(1) + (-1)^pow*dlt*ceil(pow/2);
        if bvec(2) <= 0
            bvec(2) = 0.1;
        end
        ft = gfunc_int(bvec(2),x,reg);
        rf = ftt*ft;
        pow = pow + 1;
    end    
    adj = 10000000;    
    k = 0;
    while (abs(adj) > tol)
        adj = (bvec(2) - bvec(1))/(ft - ftt)*ft;
        bnew = bvec(2) - adj;
        fnew = gfunc_int(bnew,x,reg);
        rf = fnew*ft;
        if rf < 0
            bvec(1) = bvec(2);
            bvec(2) = bnew;
        else
            bvec(2) = bnew;
        end
        ft = fnew;
        ftt = gfunc_int(bvec(1),x,reg);
        k = k + 1;
        if k == maxiter
            disp(['Convergence not reached in ' num2str(maxiter) 'iterations']);
            break
        end
    end
    niter = k;
    bml = bvec(2);
end

sml = gamma(3/bml)/gamma(1/bml)*(bml/N*sum(abs_x.^bml))^(2/bml);
end

function [g gp] = gfunc_int(bet,x,alpha)
% Nonlinear function for ML shape parameter computation

% OUTPUT
% g = function to find zeros of
% gp = analitic firs derivative of g

% INPUT
% x = input sequence
% bet = evaluation point 

x = x(:);
if alpha == 0
    abs_x = abs(x(:));
else
    abs_x = sqrt(x(:).^2+alpha^2);
end
%% -- function g
g1num = psi(1./(bet));
g1 = g1num./(bet);
g2num = sum(abs_x.^bet.*log(abs_x));
g2den = sum(abs_x.^bet);
g2 = g2num/(g2den);
g3num = log(bet/numel(x)*(g2den));
g3 = g3num/(bet);
g = 1+g1-g2+g3;
%% -- first derivative of g
gp = -g1/(bet) - psi(1,1/(bet))/(bet^3) + 1/(bet^2)- ...
    sum(abs_x.^bet.*log(abs_x).^2)./(g2den) + g2.^2 + ...
    g2/(bet) - 1/(bet)*g3;
end